@@ -1,32 +1,5 @@
-///////////////////////////////////////////////////////////////////////////
-//
-// Copyright (c) 2012-2019 DreamWorks Animation LLC
-//
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
-//
-// Redistributions of source code must retain the above copyright
-// and license notice and the following restrictions and disclaimer.
-//
-// *     Neither the name of DreamWorks Animation nor the names of
-// its contributors may be used to endorse or promote products derived
-// from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-// IN NO EVENT SHALL THE COPYRIGHT HOLDERS' AND CONTRIBUTORS' AGGREGATE
-// LIABILITY FOR ALL CLAIMS REGARDLESS OF THEIR BASIS EXCEED US$250.00.
-//
-///////////////////////////////////////////////////////////////////////////
+// Copyright Contributors to the OpenVDB Project
+// SPDX-License-Identifier: MPL-2.0
 //
 /// @file   Morphology.h
 ///
@@ -306,10 +279,7 @@ protected:
                 leaf = acc.probeLeaf(orig);
                 if ((leaf == nullptr) && !acc.isValueOn(orig)) leaf = acc.touchLeaf(orig);
             }
-#ifndef _MSC_VER // Visual C++ doesn't guarantee thread-safe initialization of local statics
-            static
-#endif
-            const int N = (LEAF_DIM - 1)*(DY + DX*LEAF_DIM);
+            static const int N = (LEAF_DIM - 1)*(DY + DX*LEAF_DIM);
             if (leaf) leaf->getValueMask().template getWord<Word>(indx-N) |= mask;
         }
 
@@ -322,10 +292,7 @@ protected:
                 leaf = acc.probeLeaf(orig);
                 isOn = leaf ? false : acc.isValueOn(orig);
             }
-#ifndef _MSC_VER // Visual C++ doesn't guarantee thread-safe initialization of local statics
-            static
-#endif
-            const int N = (LEAF_DIM -1 )*(DY + DX*LEAF_DIM);
+            static const int N = (LEAF_DIM -1 )*(DY + DX*LEAF_DIM);
             return leaf ? leaf->getValueMask().template getWord<Word>(indx-N)
                 : isOn ? ~Word(0) : Word(0);
         }
@@ -1100,7 +1067,3 @@ dilateActiveValues(tree::LeafManager<TreeType>& manager,
 } // namespace openvdb
 
 #endif // OPENVDB_TOOLS_MORPHOLOGY_HAS_BEEN_INCLUDED
-
-// Copyright (c) 2012-2019 DreamWorks Animation LLC
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
